class GroundColor < ActiveRecord::Base
  include ElementInspire
  belongs_to :panel
  belongs_to :color
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :code, :presence => true, :numericality => {:greater_than_or_equal_to => 0, :less_than => 0x1000000}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  before_validation :valid_encode
  
  def valid_encode
    ['caption'].each do |a|
      next if attributes[a] == nil
      raise Pettanr::BadRequest unless attributes[a].valid_encoding?
    end
  end
  
  def supply_default
    self.code ||= 0
    if self.panel
      self.t = self.panel.new_t 
      self.z = self.panel.new_z 
    end
  end
  
  def overwrite pid
    self.panel_id = pid
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.panel.own?(roles)
    self.panel.visible? roles
  end
  
  def opt_div_style spot = nil, opacity = 20
    o = (spot and spot != self) ? "opacity: #{opacity.to_f/100}; filter:alpha(opacity=#{opacity});" : ''
    "position: absolute; width:#{self.panel.width}px; height:#{self.panel.height}px; z-index:#{self.z}; background-color:##{format("%06x", self.code)}; #{o}"
  end
  
  def tag_id c = nil
    'panel' + tag_panel_id + 'ground_color' + tag_element_id + c.to_s
  end
  
  def field_tag_id f
    self.tag_id + f.to_s
  end
  
  def tag_panel_id
    self.panel.new_record? ? '0' : self.panel.id.to_s
  end
  
  def tag_element_id
    self.new_record? ? '0' : self.id.to_s
  end
  
  def tag_element_type
    'ground_color'
  end
  
  def field_tree f
    'panels-' + self.tag_panel_id + '-ground_colors_attributes-' + self.tag_element_id + '-' + f.to_s
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list_where
    'panels.publish > 0'
  end
  
  def self.mylist_where au
    ['panels.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['panels.author_id = ? and panels.publish > 0', au.id]
  end
  
  def self.list page = 1, page_size = self.default_page_size
    GroundColor.where(self.list_where()).includes(GroundColor.list_opt).order('ground_colors.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_ground_color_page_size
    GroundColor.where(self.mylist_where(au)).includes(GroundColor.list_opt).order('ground_colors.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_ground_color_page_size
    GroundColor.where(self.himlist_where(au)).includes(GroundColor.list_opt).order('ground_colors.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(GroundColor.where(self.list_where()).includes(GroundColor.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_ground_color_page_size
    Kaminari.paginate_array(Array.new(GroundColor.where(self.mylist_where(au)).includes(GroundColor.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_ground_color_page_size
    Kaminari.paginate_array(Array.new(GroundColor.where(self.himlist_where(au)).includes(GroundColor.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {:panel => {:author => {}} }
  end
  
  def self.list_json_opt
    {:include => {:panel => {:include => {:author => {}}} }}
  end
  
  def self.show cid, roles
    opt = {}
    opt.merge!(GroundColor.show_opt)
    res = GroundColor.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}} }}
  end
  
  def self.show_json_opt
    {:include => {:panel => {:include => {:author => {}}} }}
  end
  
  def store au
    if self.new_record?
      self.panel.ground_colors.build(self.attributes)
    else
      self.panel.ground_colors.each do |ground_color|
        next unless ground_color == self
        attr = self.attributes
        attr.delete 'id'
        ground_color.attributes = attr
        break
      end
    end
    self.panel.store({}, au)
  end
  
  def remove au
    self.panel.remove_element(self, au)
  end
  
  def scenario
    if caption.blank?
      ''
    else
      '<p>' + ERB::Util.html_escape(self.caption) + '</p>'
    end
  end
  
  def plain_scenario
    if caption.blank?
      ''
    else
      self.caption + "\n"
    end
  end
  
end
