class SpeechBalloon < ActiveRecord::Base
  include ElementInspire
  has_one :balloon, :dependent => :destroy
  has_one :speech, :dependent => :destroy
  belongs_to :speech_balloon_template
  belongs_to :panel
  
  accepts_nested_attributes_for :balloon
  accepts_nested_attributes_for :speech
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :speech_balloon_template_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :classname, :presence => true, :length => {:maximum => 50}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
#  validates :settings, :presence => true
  
  before_validation :valid_encode
  
  def valid_encode
    ['caption', 'classname', 'settings'].each do |a|
      next if attributes[a] == nil
      raise Pettanr::BadRequest unless attributes[a].valid_encoding?
    end
  end
  
  def supply_default
    if self.panel
      self.t = self.panel.new_t 
      self.z = self.panel.new_z 
    end
  end
  
  def overwrite pid
    self.panel_id = pid
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.panel.own?(roles)
    self.panel.visible? roles
  end
  
  def tag_id c = nil
    'panel' + tag_panel_id + 'speech_balloon' + tag_element_id + c.to_s
  end
  
  def field_tag_id f
    self.tag_id + f.to_s
  end
  
  def tag_panel_id
    self.panel.new_record? ? '0' : self.panel.id.to_s
  end
  
  def tag_element_id
    self.new_record? ? '0' : self.id.to_s
  end
  
  def tag_element_type
    'speech_balloon'
  end
  
  def field_tree f
    'panels-' + self.tag_panel_id + '-speech_balloons_attributes-' + self.tag_element_id + '-' + f.to_s
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list_where
    'panels.publish > 0'
  end
  
  def self.mylist_where au
    ['panels.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['panels.author_id = ? and panels.publish > 0', au.id]
  end
  
  def self.list page = 1, page_size = self.default_page_size
    SpeechBalloon.where(self.list_where()).includes(SpeechBalloon.list_opt).order('speech_balloons.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_speech_balloon_page_size
    SpeechBalloon.where(self.mylist_where(au)).includes(SpeechBalloon.list_opt).order('speech_balloons.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_speech_balloon_page_size
    SpeechBalloon.where(self.himlist_where(au)).includes(SpeechBalloon.list_opt).order('speech_balloons.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(SpeechBalloon.where(self.list_where()).includes(SpeechBalloon.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_speech_balloon_page_size
    Kaminari.paginate_array(Array.new(SpeechBalloon.where(self.mylist_where(au)).includes(SpeechBalloon.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_speech_balloon_page_size
    Kaminari.paginate_array(Array.new(SpeechBalloon.where(self.himlist_where(au)).includes(SpeechBalloon.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {:panel => {:author => {}}, :balloon => {}, :speech => {}, :speech_balloon_template => {} }
  end
  
  def self.list_json_opt
    {:include => {:panel => {:include => {:author => {} }}, :balloon => {}, :speech => {}, :speech_balloon_template => {} }}
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(SpeechBalloon.show_opt)
    res = SpeechBalloon.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :balloon => {}, :speech => {}, :speech_balloon_template => {} }}
  end
  
  def self.show_json_opt
    {:include => {:panel => {:include => {:author => {} }}, :balloon => {}, :speech => {}, :speech_balloon_template => {} }}
  end
  
  def store au, attr
    if self.new_record?
      sb = self.panel.speech_balloons.build
      sb.attributes = attr
    else
      self.panel.speech_balloons.each do |speech_balloon|
        next unless speech_balloon == self
        attr.delete 'id'
        speech_balloon.attributes = attr
        break
      end
    end
    self.panel.store({}, au)
  end
  
  def remove au
    self.panel.remove_element(self, au)
  end
  
  def parts
    @parts ||= [self.balloon, self.speech]
  end
  
  def scenario
    ERB::Util.html_escape(self.caption.to_s) + self.balloon.scenario.to_s + 
    self.speech.scenario.to_s
  end
  
  def plain_scenario
    self.caption.to_s + self.balloon.plain_scenario.to_s + 
    self.speech.plain_scenario.to_s
  end
  
end
